﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Printing;

namespace Editor
{
	public partial class FrmEditor : Form
	{
		private string _fileName;	// 记录当前正在编辑的文件名
		StringReader _srPrint;		// 记录要预览或打印的文本内容
		Font _printFont;			// 记录预览或打印的文本字体
		PageSettings _pageSettings;			// 记录默认的页面设置
		PrinterSettings _printerSettings;		// 记录默认的打印机设置

		public FrmEditor()
		{
			InitializeComponent();

			_fileName = "";
			_srPrint = null;
			_pageSettings = new PageSettings();
			_printerSettings = new PrinterSettings();
		}

		private void miFileExit_Click(object sender, EventArgs e)
		{
			Application.Exit();
		}

		private void miFileNew_Click(object sender, EventArgs e)
		{
			rtbEditor.Clear();

			_fileName = "";
		}

		private void miFileOpen_Click(object sender, EventArgs e)
		{
			// 指定文件类型选择项
			dlgOpen.Filter = "文本文件(*.txt)|*.txt|富格式文件(*.rtf)|*.rtf|所有文件(*.*)|*.*";
			if (dlgOpen.ShowDialog() == DialogResult.OK)
			{
				_fileName = dlgOpen.FileName;
				// 如果扩展名为".rtf"，就保存为富格式文件，否则保存为普通文本文件
				if (Path.GetExtension(_fileName) == ".rtf")
					rtbEditor.LoadFile(_fileName, RichTextBoxStreamType.RichText);
				else
					rtbEditor.LoadFile(_fileName, RichTextBoxStreamType.PlainText);
			}
		}

		private void miFileSave_Click(object sender, EventArgs e)
		{
			// 如果文件名为空，表示是新文件
			if (_fileName == "")
			{
				// 动态创建一个文件保存对话框
				SaveFileDialog dlgSave = new SaveFileDialog();
				// 设置保存对话框的默认参数
				// 指定默认的对话框标题
				dlgSave.Title = "保存文本编辑器文件";
				// 指定默认的文件路径
				DirectoryInfo di = new DirectoryInfo(Application.StartupPath);
				di = di.Parent.Parent;
				dlgSave.InitialDirectory = di.FullName;
				dlgSave.Filter = "文本文件(*.txt)|*.txt|富格式文件(*.rtf)|*.rtf|所有文件(*.*)|*.*";
				// 显示保存文件对话框，且用户按下了“确认”按钮
				if (dlgSave.ShowDialog() == DialogResult.OK)
				{
					_fileName = dlgSave.FileName;
					if (Path.GetExtension(_fileName) == ".rtf")
						rtbEditor.SaveFile(_fileName, RichTextBoxStreamType.RichText);
					else
						rtbEditor.SaveFile(_fileName, RichTextBoxStreamType.PlainText);
				}
			}
			// 否则表示是旧文件
			else
			{
				if (Path.GetExtension(_fileName) == ".rtf")
					rtbEditor.SaveFile(_fileName, RichTextBoxStreamType.RichText);
				else
					rtbEditor.SaveFile(_fileName, RichTextBoxStreamType.PlainText);
			}
		}

		private void miFileSaveAs_Click(object sender, EventArgs e)
		{
			SaveFileDialog dlgSave = new SaveFileDialog();

			dlgSave.Title = "保存文本编辑器文件";
			DirectoryInfo di = new DirectoryInfo(Application.StartupPath);
			di = di.Parent.Parent;
			dlgSave.InitialDirectory = di.FullName;
			dlgSave.Filter = "文本文件(*.txt)|*.txt|富格式文件(*.rtf)|*.rtf|所有文件(*.*)|*.*";
			if (dlgSave.ShowDialog() == DialogResult.OK)
			{
				_fileName = dlgSave.FileName;
				if (Path.GetExtension(_fileName) == ".rtf")
					rtbEditor.SaveFile(_fileName, RichTextBoxStreamType.RichText);
				else
					rtbEditor.SaveFile(_fileName, RichTextBoxStreamType.PlainText);
			}
		}

		private void miEditCut_Click(object sender, EventArgs e)
		{
			rtbEditor.Cut();
		}

		private void miEditCopy_Click(object sender, EventArgs e)
		{
			rtbEditor.Copy();
		}

		private void miEditPaste_Click(object sender, EventArgs e)
		{
			rtbEditor.Paste();
		}

		private void miEditUnDo_Click(object sender, EventArgs e)
		{
			rtbEditor.Undo();
		}

		private void miEditReDo_Click(object sender, EventArgs e)
		{
			rtbEditor.Redo();
		}

		private void miStyleFont_Click(object sender, EventArgs e)
		{
			// 创建字体对话框
			FontDialog dlgFont = new FontDialog();
			dlgFont.ShowColor = true;	// 在颜色对话框中是否显示颜色
			dlgFont.ShowApply = true;	// 在颜色对话框中是否显示“应用”按钮

			// 设置字体对话框的默认字体，
			// 如果已经有选择文本，则设置为选择文本的字体和颜色
			// 否则设置为全文的字体和颜色
			if (rtbEditor.SelectionLength > 0)
			{
				dlgFont.Font = rtbEditor.SelectionFont;
				dlgFont.Color = rtbEditor.SelectionColor;
			}
			else
			{
				dlgFont.Font = rtbEditor.Font;
				dlgFont.Color = rtbEditor.ForeColor;
			}
			// 显示字体对话框，且用户按下了“确认”按钮
			if (dlgFont.ShowDialog() == DialogResult.OK)
			{
				// 如果有选择文本，刚修改选择文本的字体和颜色
				if (rtbEditor.SelectionLength > 0)
				{
					rtbEditor.SelectionFont = dlgFont.Font;
					rtbEditor.SelectionColor = dlgFont.Color;
				}
				// 否则修改整个文本的字体和颜色
				else
				{
					rtbEditor.Font = dlgFont.Font;
					rtbEditor.ForeColor = dlgFont.Color;
				}
			}
		}

		private void miStyleColor_Click(object sender, EventArgs e)
		{
			ColorDialog dlgColor = new ColorDialog();
			if (rtbEditor.SelectionLength > 0)
				dlgColor.Color = rtbEditor.SelectionColor;
			else
				dlgColor.Color = rtbEditor.ForeColor;

			if (dlgColor.ShowDialog() == DialogResult.OK)
			{
				if (rtbEditor.SelectionLength > 0)
					rtbEditor.SelectionColor = dlgColor.Color;
				else
					rtbEditor.ForeColor = dlgColor.Color;
			}
		}

		private void miStyleAlignLeft_Click(object sender, EventArgs e)
		{
			rtbEditor.SelectionAlignment = HorizontalAlignment.Left;
		}

		private void miStyleAlignCenter_Click(object sender, EventArgs e)
		{
			rtbEditor.SelectionAlignment = HorizontalAlignment.Center;
		}

		private void miStyleAlignRight_Click(object sender, EventArgs e)
		{
			rtbEditor.SelectionAlignment = HorizontalAlignment.Right;
		}

		private void miStyleIndentLeft_Click(object sender, EventArgs e)
		{
			rtbEditor.SelectionIndent = 20;
		}

		private void miStyleIndentRught_Click(object sender, EventArgs e)
		{
			rtbEditor.SelectionRightIndent = 20;
		}

		private void miStyleIndentFirst_Click(object sender, EventArgs e)
		{
			rtbEditor.SelectionIndent = 20;
			rtbEditor.SelectionHangingIndent = -20;
		}

		private void miStyleIndentSecond_Click(object sender, EventArgs e)
		{
			rtbEditor.SelectionHangingIndent = 20;
		}

		private void docPrint_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
		{
			float line = 0;		// 待输出的总行数
			float yPos = 0;		// 输出每行时左上角上边位置（象素）
			int count = 0;		// 当前页的当前行数
			float leftMargin = e.MarginBounds.Left;		// 输出页面的左边界
			float topMargin = e.MarginBounds.Top;		// 输出页面的上边界
			string sLine = null;	// 待输出的文本行
			// 计算每页的行数
			line = (e.MarginBounds.Bottom - e.MarginBounds.Top) / rtbEditor.Font.GetHeight(e.Graphics);
			// 循环打印每一页
			while (count < line && ((sLine = _srPrint.ReadLine()) != null))
			{
				yPos = topMargin + (count * _printFont.GetHeight(e.Graphics));
				e.Graphics.DrawString(sLine, _printFont, Brushes.Black, leftMargin, yPos, new StringFormat());
				count++;
			}
			// 设置是否还有要输出的打印行
			if (sLine != null)
				e.HasMorePages = true;
			else
				e.HasMorePages = false;
		}

		private void miPrintPreview_Click(object sender, EventArgs e)
		{
			_srPrint = new StringReader(rtbEditor.Text);
			_printFont = rtbEditor.Font;

			// 创建打印预览对话框
			PrintPreviewDialog dlgPreview = new PrintPreviewDialog();
			// 使用用户指定的页面设置
			docPrint.PrinterSettings = _printerSettings;
			docPrint.DefaultPageSettings = _pageSettings;
			// 设置打印文档的名称
			docPrint.DocumentName = Path.GetFileName(_fileName);
			// 指定打印文档:将预览对话框与打印文档控件关联起来
			dlgPreview.Document = docPrint;
			try
			{
				// 预览
				dlgPreview.ShowDialog();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "打印出错", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void miPrintPrint_Click(object sender, EventArgs e)
		{
			// 为打印指定数据和字体
			_srPrint = new StringReader(rtbEditor.Text);
			_printFont = rtbEditor.Font;

			// 创建打印对话框
			PrintDialog dlgPrint = new PrintDialog();
			// 使用用户指定的页面设置
			docPrint.PrinterSettings = _printerSettings;
			docPrint.DefaultPageSettings = _pageSettings;
			// 设置打印文档的名称
			docPrint.DocumentName = Path.GetFileName(_fileName);
			// 指定打印文档:将预览对话框与打印文档控件关联起来
			dlgPrint.Document = docPrint;
			// 打印
			if (dlgPrint.ShowDialog() == DialogResult.OK)
				docPrint.Print();

		}

		private void miPrintPageSetup_Click(object sender, EventArgs e)
		{
			PageSetupDialog dlgPageSetup = new PageSetupDialog();
			dlgPageSetup.PageSettings = _pageSettings;
			if (dlgPageSetup.ShowDialog() == DialogResult.OK)
				_pageSettings = dlgPageSetup.PageSettings;
		}

		private void miPrintPrinterSetup_Click(object sender, EventArgs e)
		{
			PageSetupDialog dlgPageSetup = new PageSetupDialog();
			dlgPageSetup.PageSettings = _pageSettings;
			dlgPageSetup.PrinterSettings = _printerSettings;
			if (dlgPageSetup.ShowDialog() == DialogResult.OK)
				_pageSettings = dlgPageSetup.PageSettings;

		}
	}
}
